package anywheresoftware.agraham.b4j.jxfunction;

import com.excelsior.xFunction.Argument;
import com.excelsior.xFunction.Pointer;
import com.excelsior.xFunction.Structure;
import com.excelsior.xFunction.xFunction;

import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.DependsOn;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;


/**
 *xFunction is a lightweight (100KB) library that enables access to the native API.
 *Two versions are available - Windows and Linux. This library uses the Windows version.
 *
 *Although not as optimised for performance as the heavyweights, JNA and BridJ, this library is convenient to use
 *for occasional calls to the native API.
 *
 *To use the xFunction object you must have a copy of xFunction.jar in your Additional Libraries folder
 *and place a copy of xFunction.dll in your Objects folder or on your Java classpath.
 */
@DependsOn(values = { "xFunction" })
@Author("Andrew Graham")
@Version(1.0f)
@ShortName("xFunction")
public class xFunctionWrapper {
	// 1.0 	initial release

	protected static final double version = 1.0;
	
	/**
	 * Returns the version number of the library.
	 */
	public double getVersion() {
		return version;
	}

	/**
	 *The xFunction library allows you to call external functions, implemented in any language and residing in DLLs
	 *or shared libraries, from your Java program. With xFunction, you do not need to implement native methods operating
	 *through Java Native Interface (JNI). Instead, you extend and instantiate xFunction classes to create conventional
	 *Java objects representing external functions, data structures, pointers, and callbacks. All necessary data conversions
	 *and external function calls are done seamlessly by the xFunction library.
	 *
	 *xFunction is a product ofExcelsior, LLC http://www.excelsior-usa.com and has been placed in the public domain.
	 *It may be downloaded from http://www.excelsior-usa.com/xfunctiondl.html, taking note of the conditions of use.
	 *
	 *It is not really practical to add new API calls from within B4J code so the source of this library is included in the
	 *library archive so you can modify yourself and compile it with Eclipse or the Simple Library Compiler.
	 */
	public void LIBRARY_DOC() {
	}
	
	// API call definitions, saved so only called once for efficiency
	xFunction bBeepii ;
	xFunction bMsgBeepu;
	xFunction iGetVerExAp;
	
	
	/**
	 *Initializes the API call functions.
	 *
	 *If you are writing your own function definitions note that the function parameter type definition requires 
	 *the fully qualified name, as used for Class.forName() except for the following
	 *"void", "byte", "char", "short", "int", "long", "float", "double", "CSTRING", "UNICODESTRING" and "BSTR".
	 */
	public void Initialize() throws Exception {
		 bBeepii = new xFunction("kernel32", "int Beep(int,int)");			 
		 bMsgBeepu = new xFunction("User32", "int MessageBeep(int)");
		 iGetVerExAp=new xFunction("kernel32", "int GetVersionExA(anywheresoftware.agraham.b4j.jxfunction.OSVERSIONINFO*)");

	}	
	
	/**
	 *Generates simple tones on the speaker. The function is synchronous.
	 *It performs an alertable wait and does not return control to its caller until the sound finishes.
	 *
	 *dwFreq : The frequency of the sound, in hertz. This parameter must be in the range 37 through 32,767 
	 *dwDuration : The duration of the sound, in milliseconds. 
	 *Returns True if the call succeeds.
	 *
	 */
	public boolean Beep(int dwFreq , int dwDuration) throws Exception  {
		Integer r = (Integer) bBeepii.invoke(new Argument(dwFreq),new Argument(dwDuration));
		return r != 0 ? true : false;
	}
	
	
	/**
	 *Plays a waveform sound. The waveform sound for each sound type is identified by an entry in the registry.
	 *The beeptype parameter can be one of the following values.
	 *
	 *MB_OK : 0x00 The sound specified as the Windows Default Beep sound.
	 *MB_ICONERROR0 : x10 The sound specified as the Windows Critical Stop sound.
	 *MB_ICONQUESTION : 0x20 The sound specified as the Windows Question sound.
	 *MB_ICONWARNING : 0x30 The sound specified as the Windows Exclamation sound.
	 *MB_ICONINFORMATION : 0x40 The sound specified as the Windows Asterisk sound.
	 */
	public boolean MessageBeep(int beeptype) throws Exception { 
		Integer r = (Integer) bMsgBeepu.invoke(new Argument(beeptype));
		return r != 0 ? true : false;	
	}
	
	
	/**
	 *Returns a string describing the current OS.
	 */
	public String GetVersionExA() throws Exception { 
		OSVERSIONINFO info = new OSVERSIONINFO();
        info.dwOSVersionInfoSize = 148;//Size of OSVERSIONINFO
        Pointer arg = Pointer.createPointerTo(info);
        iGetVerExAp.invoke(arg);
        info = (OSVERSIONINFO)arg.deref();
        StringBuilder s = new StringBuilder();        
        s.append("Major Version: "+ info.dwMajorVersion).append("\r\n");
        s.append("Minor Version: "+ info.dwMinorVersion).append("\r\n");
        s.append("Build Number: "+ info.dwBuildNumber).append("\r\n");
        s.append("Platform Id: " + info.dwPlatformId).append("\r\n");
        s.append("CSD Version: " + new String(info.szCSDVersion));
        return s.toString();
	}
	
	
}	
	
class OSVERSIONINFO extends Structure{

	int dwOSVersionInfoSize;
    int dwMajorVersion;
    int dwMinorVersion;
    int dwBuildNumber;
    int dwPlatformId;
    char[] szCSDVersion;

    public String defineLayout(){
        return "int dwOSVersionInfoSize,int dwMajorVersion,int dwMinorVersion,"+
               "int dwBuildNumber,int dwPlatformId, char[128] szCSDVersion";
    }

    public OSVERSIONINFO(){
        szCSDVersion = new char[128];
    }
}
	
	
	

